// dialog.js
// Generic code to show messages with a dismiss button
// There are two functions available.
//
// function showDialog(text, callback /* optional */)
//    Shows the text in a dialog. It has an optional callback argument.
//    If the optional callback is provided, it is called when the OK button is clicked.
//
//  async function showDialogAsync(text)
//    Shows the text in a dialog. This returns a promise, which is resolved to true
//    when the OK button is clicked. This form is thenable can be used in an await statement.

import $ from "./jquery.module.js";
import {convertDisplayNameToID} from "./regularize_string.js";

// Looks for an element with an ID of wrapper, which should be the top level div in all blocks ;
function getDialogWrapper() {
    let dialogWrapper = $("#wrapper");
    if (dialogWrapper.length === 0) {
        dialogWrapper = $(".dialog-wrapper");
    }
    return dialogWrapper;
}

// When the okbutton is clicked, the dialog box will be removed and the call back executed, which is usually to go to a new page ;
function okButton(evt, callback) {
    $(".dialog7gen_holder").remove();
    callback?.();
}

function okButtonResolve(evt, resolve) {
    $(".dialog7gen_holder").remove();
    resolve(true);
}

// CREATES A DIV THAT HAS THE MESSAGE PASSED TO IT AND AN OK BUTTON ;
function dialogMessageHTML(text) {
    return `
        <div class="dialog7gen_holder container">
            <div class="dialog7gen">
                <div class="message7gen">
                    ${text}
                </div>
                <div class="okbuttonrow">
<!--                    <button class="okbutton7gen"><img src="../../game_theme/theme/icons/next.png" alt="arrow pointing right"</button>-->
                    <button class="okbutton7gen">OK</button>
                </div>
                <div class="choice-button-row"></div>
            </div>
        </div>
    `;
}

function thread () {
    let resolve, reject;
    const promise = new Promise((res, rej) => {
        resolve = res;
        reject = rej;
    })
    return {promise, resolve, reject};
}

function showDialog(text, callback /* optional */) {
    const $wrapper = getDialogWrapper();
    $wrapper.append(dialogMessageHTML(text));
    $(".choice-button-row").hide();
    $(".okbutton7gen").on("click", (evt)=>okButton(evt, callback));
}

async function showDialogAsync(text) {
    const $wrapper = getDialogWrapper();
    $wrapper.append(dialogMessageHTML(text));
    $(".choice-button-row").hide();

    return new Promise((resolve, reject) => {
        $(".okbutton7gen").on("click", (evt) => okButtonResolve(evt, resolve));
    });
}

async function showChoiceDialogAsync(message, choices) {
    // "message" is displayed at the top of the dialog
    //
    // "choices" is an array of strings that are used as labels for buttons
    // at the bottom of the dialog. When a button is pressed, the function returns
    // the string corresponding to the clicked button.

    const $wrapper = getDialogWrapper();
    $wrapper.append(dialogMessageHTML(message));
    $(".okbuttonrow").hide();

    const $buttonRow = $(".choice-button-row");
    const {promise, resolve} = thread();

    for (const choice of choices) {
        const btnClass = convertDisplayNameToID({prefix: "btnClass", displayName: choice});
        $buttonRow.append(`<button class="${btnClass}">${choice}</button>`);
        $(`.${btnClass}`).on("click", (evt) => resolve(choice));
    }

    const answer = await promise;
    $(".dialog7gen_holder").remove();
    return answer;
}

function showPersistentMessage(message) {
    // This dialog stays up until you dismiss it. The dismiss function
    // is returned.
    const $wrapper = getDialogWrapper();
    $wrapper.append(dialogMessageHTML(message));
    $(".choice-button-row").hide();
    $(".okbuttonrow").hide();

    return () => $(".dialog7gen_holder").remove();
}

export {showDialog, showDialogAsync, showChoiceDialogAsync, showPersistentMessage};
