import $ from "./jquery_no_ui.module.js";
import SetConfig from "./set_config_class.js";
import * as audio from "./audio.js";
import {setAudioFile} from "./audio.js";
let options;
let targets_config;
let targets ;
let numberOfGoodTargetsHit = 0;
let goodTargetLimit ;
const targetLimit = 1000;
let targetSlots; // Vertical height for targets to be released
let targetTypes = [];
let targetProps = {};
let targetsSpawned = 0;
let targetInterval = 700; // ms
let finished = false;
let setConfigHandler = new SetConfig();

function setConfig(config) {
    setConfigHandler.setConfig(config);
}


function initializeMovingTargets() {

    targetSlots = targets_config.target_slots;

    for (const target of targets) {
        const targetType = target.type;
        targetTypes.push(targetType);
        targetProps[targetType] = {
            image: target.image,
            value: target.value
        }
    }
}

// Define visible plugin
$.fn.visible = function(isVisible = true) {
    return this.css("visibility", isVisible ? "visible" : "hidden");
}

function getRandomInt(max) {
    return Math.floor(Math.random() * max);
}

function spawnTargets() {
    spawnOneTarget();
    ++targetsSpawned;
    if (targetsSpawned < targetLimit && numberOfGoodTargetsHit < goodTargetLimit) {
        setTimeout(spawnTargets, targetInterval);
    }
}

function spawnOneTarget() {
    const targetType = targetTypes[getRandomInt(targetTypes.length)];
    const nextSlot = getRandomInt(targets_config.target_slots.length);
    addTarget(targetType, nextSlot);
}

function hitGoodTarget(e) {
    if (finished) {
        return;
    }
    setAudioFile(targets_config.hitGoodTargetSound) ;
    audio.play(true) ;
    $(e.delegateTarget).visible(false);
    updateScore(++numberOfGoodTargetsHit);
    if (numberOfGoodTargetsHit >= goodTargetLimit) {
        success();
    }
}

function hitBadTarget(e) {
    if (finished) {
        return;
    }
    setAudioFile(targets_config.hitBadTargetSound) ;
    audio.play(true) ;
    $(e.delegateTarget).visible(false);
}

function updateScore(newScore) {
    $("#score").text(`${options.scoreLabel} ${newScore}`);
}

function addTarget(targetType, slot) {
    let $target = $('<div/>', {
        "class": `animated-target ${targetType} target`,
        "draggable": false,
    });
    $target.css("user-select", "none");
    $target.appendTo($("#game-board-overlay"));
    $target.html(`<img src="${targetProps[targetType].image}" draggable="false">`);
    $target.click(targetProps[targetType].value === "good" ? hitGoodTarget : hitBadTarget);
    let topPosition = `${targetSlots[slot]}vh`;
    $target.css("top", topPosition);
    $target.on("animationend", () => $target.remove());
}

function success() {
    finished = true;
    let $success = $("#success");
    $success.visible();
    $success.text(options.successText);

    $("#next").visible().click(() => {
        window.location.href= options.completionURL;
    });
}

function adjustForImageSize() {
    let width = $("#game-board-img").css("width");
    let $toolbar = $("#toolbar");
    if ($toolbar.length > 0) {
        $toolbar.width(width);
    }
}

$(window).on("resize", ()=>{
    positionOverlayOverImage();
    adjustForImageSize();
});

function positionOverlayOverImage() {
    let $overlay = $("#game-board-overlay");
    let $image = $("#game-board-img");
    $overlay.width($image.width()).height($image.height()).position($image.position());
}

// let targetsConfigPromise = new Promise((resolve,reject) => resolveTargetConfig = resolve);


function loadGameBoardImage(config) {
    let $img = $("#game-board-img");
    return new Promise((resolve, reject) => {
        $img.on("load",  () => resolve(config)); // Pass config to next "then" function
        $img.on("error", () => reject());
        $img.attr("src", config.targets_config.game_board_image);
    });
}

$(() => {
    setConfigHandler.then (loadGameBoardImage). then((config)=>{
        options = config.options ;
        targets_config = config.targets_config ;
        targets = targets_config.targets;
        goodTargetLimit = targets_config.goodTargets ;
        const instructDiv = document.getElementById("instructions_help");
        instructDiv.innerHTML = config.options.instructionsPage1 ;


            const ul = document.createElement('ul');
            for (let i =0; i < targets.length ; i++){

                const listItem = document.createElement('li');

                const img = document.createElement("img");
                img.src = targets[i].image ;

                listItem.textContent = " " + targets[i].type.replace("_", " ");

                listItem.prepend(img);
                ul.appendChild(listItem) ;

            }
            instructDiv.appendChild(ul) ;

        $("#play").on("click", ()=>{
            $("#instructions1").addClass('d-none') ;
            $("#score").removeClass('d-none') ;
            initializeMovingTargets();


            // targetInterval is given a default value when declared.
            // Here we set it to the value specified in the config file;
            // if is not defined there we use the original default value;
            targetInterval = targets_config?.target_interval ?? targetInterval;

            positionOverlayOverImage();
            adjustForImageSize();

            $("#game-board-img")
            updateScore(numberOfGoodTargetsHit);
            for(const tgt of targets) {
                tgt.imageClass = "." + tgt.type  + " img" ;
                if (tgt.value === "good") {
                    $(tgt.imageClass).on("click", function() {
                        hitGoodTarget($(this));
                    });
                }
                else {
                    $(tgt.imageClass).on("click", function() {
                        hitBadTarget($(this));
                    });
                }
            }

            spawnTargets();
        });
        });

});

export {setConfig};