import $ from "./jquery.module.js";
import SetConfig from "./set_config_class.js";
import * as dialog from "./dialog.js";
import * as audio from "./audio.js";

const runner_config = new SetConfig();
let config = null;
let options = null;

let debugFlag = false;
let inset = 10;
let obstacleStagePosition = 1200 ;
let collisionCount = 0;
let newObstacleCounter = 0;
let obstacleCounter = 0;
let lastObstacleCollision = -1;
let lastObstacleRemoved = -1;
let obstaclesAvailable = 1;
const newObstacleTime = 20;
let paused = false;
let end_game = false;
let currentSetIntervalID = null;
let obstacles_exclude_region = null;

function getRandomInt(max) {
    return Math.floor(Math.random() * max);
}

function setConfig(theConfig) {
    runner_config.setConfig(theConfig);
}
function initiatePlayerJump() {
    console.log("Called initiatePlayerJump");
    $(":root").css("--jump-height",`-${options.runner.jump_height}`);
    $("#runner").addClass("runner-animation").on("animationend", finishPlayerJump);
}

function handleAnimationLoop(evt) {
    $(".obstacle")
        .filter((index,element)=>{return $(element).attr("data-obstacle") !== "goal";})
        .remove();
}
function finishPlayerJump() {
    $("#runner").removeClass("runner-animation");
}

function applyScrollingImageCSS() {
    $(".scrolling-image").css({
        background: `url("${options.scrolling_image.image_path}") repeat-x`,
        "background-size": "contain",
        width: `${options.scrolling_image.width}`,
        height: `${options.scrolling_image.height}`
    });
    $("#runner").css({
        width: `${options.scrolling_image.width}`,
        height: `${options.scrolling_image.height}`
    });
    $("#runner img").css({
        width: `auto`,
        height: `${options.runner.height}`
    });
}
function initialSetup() {
    $(document).on("keydown", (evt) => {
        evt.preventDefault();
        if(evt.key === " ") {
            initiatePlayerJump();
        }
    });
    if ( options.obstacle_stage_position ) {
        obstacleStagePosition = options.obstacle_stage_position ;
    }




    applyScrollingImageCSS();
    $(".scrolling-image").on("animationiteration", handleAnimationLoop);

    $("#runner")
        .append(`<img id="runner-image" src="${options.runner.image}" alt="runner-image">`);

    // The following assumes "options.runner.location" will contain an object that jQuery's css function
    // will recognize as valid CSS to specify the runner's absolute position
    $("#runner img").css(options.runner.location).css({
        width: "auto",
        height: options.runner.height
    });

    if ("collision_rect_inset" in options) {
        inset = options.collision_rect_inset;
    }

    if ("collision_counter_label" in options) {
        $("#collision-counter-label").html(`${options.collision_counter_label}`);
    }

    obstaclesAvailable = config.obstacles.length;
}

// PAUSES THE ANIMATIONS -used when hint button is clicked ;
function pauseAnimations(evt) {
    $(".scroll_animate").css("animation-play-state", "paused");
    $(".runner-animation").css("animation-play-state", "paused");
    paused = true;
}
// RESUMES ANIMATIONS  - used when hint dialog box is closed ;
function resumeAnimations(evt) {
    $(".scroll_animate").css("animation-play-state", "running");
    $(".runner-animation").css("animation-play-state", "running");
    paused = false;
}

// CHECKS TO SEE IF TWO RECTANGLES OVERLAP ;
function rectsOverlap(domRect1, domRect2, infinite_height = false)
{
    // console.log(domRect1, domRect2);
    // If one rectangle is on left side of other
    if (domRect1.left > domRect2.right - inset || domRect2.left > domRect1.right - inset) {
        return false;
    }

    if (infinite_height)
        return true;

    // If one rectangle is above other
    if (domRect1.top > domRect2.bottom - inset || domRect2.top > domRect1.bottom - inset) {
        return false;
    }

    return true;
}


/*
   COUNTS OBSTACLE COLLISIONS. IF OBSTACLE IS NOT THE GOAL, INCREMENTS NUMBER OF COLLISIONS AND
   PLAYS AUDIO FOR COLLISION .
   CALLED EVERY 10 MILLISECONDS BY SET INTERVAL FUNCTION BELOW
 */
function incrementCollisionCount($collider) {
    let obstacleData = $collider.attr("data-obstacle");
    if (obstacleData === "goal") {
        // Was not an obstacle, do not count
        return;
    }
    let obstacleNum = Number(obstacleData);
    if (obstacleNum <= lastObstacleCollision) {
        return;
    }
    // If here, we collided with a new obstacle
    lastObstacleCollision = obstacleNum;
    ++collisionCount;
    $("#collision-count").html(collisionCount.toString());
    audio.playAudioFile(options.bad_sound);
}

function showStageInfo() {
    let pos = $(".scroll_animate")[0].getBoundingClientRect().left;
    let obs_pos = "0";
    let $obs = $(".obstacle");
    if ($obs.length > 0) {
        obs_pos = $obs[0].getBoundingClientRect().left.toString();
    }
    $("#stage-info").html(pos.toString() + "\n" + obs_pos);
}
// GETS THE DATA VALUE FROM AN OBSTACLE IN DATA-OBSTACLE AND RETURNS IT AS A NUMBER ;
function getObstacleNumber($obs) {
    return Number($obs.attr("data-obstacle"));
}

function checkExcludedRegions(left) {
    const obstacles_exclude_region = config.obstacles_exclude_region ?? null;
    if (obstacles_exclude_region === null) {
        return true;
    }

    for (const region of obstacles_exclude_region) {
        if (left < region[0]) {
            return true;
        }
        else if (region[0] <= left && left <= region[1]) {
            return false;
        }
    }

    return true;
}

/*
    GENERATES NEW OBSTACLES ON THAT STAGE (SCROLLING IMAGE BACKGROUND)
    COUNTS NUMBER OF OBSTACLES CREATED
    IF AN OBSTACLE HAS MOVED TO THE LEFT OF THE CONTAINING RECTANGLE OF THE SCROLLING IMAGE,
    OR IF WE ARE TO THE LAST OBSTACLE, THE OBSTACLE IS REMOVED.
    IF AT THE LAST OBSTACLE, A NEW 'OBSTACLE' IS PUT UP WITH THE DATA-OBSTACLE VALUE OF 'GOAL'
    AND THE SOURCE FOR THE GOAL IMAGE, OTHERWISE OBSTACLES ARE PUT UP IN THE ORDER SPECIFIED IN THE CONFIG FILE,
    E.G, IF TWO OBSTACLE IMAGES, THE SECOND WOULD BE SHOWN EVERY SECOND OBSTACLE
 */
function handleNewObstacle() {
    if (paused) {
        return;
    }

    // Determine if we are in the "dead zone," i.e. the portion of the
    // stage that will disappear when the animation loops. We make sure not
    // to deposit any new obstacles when in this zone, as those obstacles would
    // appear to disappear when the loop happens.
    let stageLeft = $(".scroll_animate")[0].getBoundingClientRect().left;
    if (stageLeft < options.stage.width - options.end_of_animation) {
        return;
    }

    // If we have reached the end of game, no more obstacles will be generated.
    if (end_game) {
        return;
    }

    ++newObstacleCounter;

    // Remove any obstacles that scrolled offscreen
    // We also check for older obstacles than this to catch the ones
    // that did not scroll off before the stage animation was reset
    const containerLeft = $(".scrolling-image-container")[0].getBoundingClientRect().left;
    $(".obstacle").each((i, element)=>{
        let right = element.getBoundingClientRect().right;
        let $obs = $(element);
        if (right <  containerLeft) {
            lastObstacleRemoved = getObstacleNumber($obs);
            $obs.remove();
        }
        else if (lastObstacleRemoved > -1) {
            if (getObstacleNumber($obs) <= lastObstacleRemoved) {
                $obs.remove();
            }
        }
    });

    if (newObstacleCounter > newObstacleTime) {
        let stageLeft =  options.obstacle_stage_position - $(".scroll_animate")[0].getBoundingClientRect().left;

        if (!checkExcludedRegions(stageLeft)) {
            return;
        }
        newObstacleCounter = 0;
        ++obstacleCounter;
        setProgress({progress: obstacleCounter, maxProgress: options.obstacle_end_game_count});
        if (obstacleCounter > options.obstacle_end_game_count) {

            if (options.goal_image) {
                const $obstacle = $(`<img class="obstacle" data-obstacle="goal" src="${options.goal_image.image}">`);
                let stageLeft =  obstacleStagePosition - $(".scroll_animate")[0].getBoundingClientRect().left;

                $obstacle.css({bottom: options.ground_level, left: stageLeft, height: options.goal_image.height, width: options.goal_image.width});
                $(".scrolling-image").append($obstacle);
            }
            end_game = true;
        }
        else {
            let idx = options.randomize_obstacles ? getRandomInt(obstaclesAvailable)  : obstacleCounter % obstaclesAvailable;
            const obstacles = config.obstacles;
            const $obstacle = $(`<img class="obstacle" data-obstacle="${obstacleCounter.toString()}" src="${obstacles[idx].image}">`);


            $obstacle.css({bottom: options.ground_level, left: stageLeft, height: obstacles[idx].height, width: obstacles[idx].width});
            $(".scrolling-image").append($obstacle);
        }

    }
}

function setProgress({progress, maxProgress}) {
    let p = 100*progress/maxProgress;
    $(".progress-bar").css({width: `${p}%`}).attr("aria-valuenow", p);
}

function testForRunnerCollision() {
    //Initialize return values foundCollision and collider
    let foundCollision = false; // Meaning no collision
    let collider = null;
    let obstacleData = null

    let runnerRect = $("#runner img")[0].getBoundingClientRect();
    if (debugFlag) {console.log("runner rect:", runnerRect);}

    $(".obstacle").each((i, element) => {
        obstacleData = $(element).attr("data-obstacle");
        let obsRect = element.getBoundingClientRect();
        if (debugFlag) {console.log("obs rect " + i.toString(), obsRect);}
        if (rectsOverlap(runnerRect, obsRect, obstacleData === "goal")) { // Guarantee we will collide if it is a goal object
            // console.log("Found collision");
            debugFlag = false;
            foundCollision = true;
            collider = $(element);
        }
    });

    // console.log("Did not find collision");
    debugFlag = false;
    return {foundCollision, collider, obstacleData};
}

// FUNCTION TO RESTART GAME AFTER THERE HAVE BEEN TOO MANY COLLISIONS (PLAYER LOST) ;
function restartGame() {
    $("#top-container").show();
    $("#collision-counter").show();
    $("#progress").show();
    collisionCount = 0;
    obstacleCounter = 0;
    lastObstacleCollision = -1;
    lastObstacleRemoved = -1;
    newObstacleCounter = 0;
    $("#collision-count").html(collisionCount.toString());
    setProgress({progress: obstacleCounter, maxProgress: options.obstacle_end_game_count});
    $(".obstacle").remove();
    // clearInterval(currentSetIntervalID);
    startEventLoop();
}

function endTheGame () {
    $("#top-container").hide();
    $("#collision-counter").hide();
    $("#progress").hide();
    audio.playAudioFile(options.good_sound);
    if (options.exit_text) {
        dialog.showDialog(
            options.exit_text,
            () => {window.location.href = options.completionURL});
    }
    else {
        // If exit_text is missing, go directly to completion_url, which should have some text about finishing the game
        window.location.href = options.completionURL ;
    }
}

function debugMode(debugFlag) {
    if (!debugFlag) {
        $(".debug_buttons").hide();
    }
}

function startEventLoop() {
    currentSetIntervalID = setInterval(async () => {
            let {foundCollision, collider, obstacleData} = testForRunnerCollision();
            if (foundCollision) {
                console.log("There was a collision!");
                // Check for end of game
                if (obstacleData === "goal") {
                    endTheGame();
                    return;
                }
                incrementCollisionCount(collider);
                if (collisionCount >= options.collision_limit) {
                    $("#top-container").hide();
                    $("#collision-counter").hide();
                    $("#progress").hide();

                    clearInterval(currentSetIntervalID); // It is important to call this _before_ entering the
                                                         // modal dialog so that the interval doesn't continue
                                                         // to run while the dialog is up.

                    await dialog.showDialogAsync(options.too_many_collisions_text);
                    restartGame();
                }
            }
            handleNewObstacle();
            showStageInfo();
        },
        100);
}

$(async () => {
    config = await runner_config.configPromise;
    options = config.options;
    obstacles_exclude_region = config?.obstacles_exclude_region;
    if ( options.startBackground ) {
        console.log("bob");
        $("#wrapper").css("background-image", "url(" + options.startBackground + ")");
    }

    // Set up buttons
    $("#debug").on("click", (evt) => {debugFlag = true;});
    $("#pause-animations").on("click", pauseAnimations);
    $("#resume-animations").on("click", resumeAnimations);
    $("#top-container").hide();
    $("#collision-counter").hide();
    $("#progress").hide();
    debugMode(false);

    await dialog.showDialogAsync(options.intro_text);
    $("#top-container").show();
    $("#collision-counter").show();
    $("#progress").show();
    initialSetup();
    startEventLoop();
});

export default setConfig;